import { Request, Response } from "express";
import logger from "../logger/logger";
import util from "util";
import { exec } from "child_process";
import fs from "fs-extra";
import { v4 as uuidv4 } from "uuid";
import Website from "../models/website";

const execPromise = util.promisify(exec);

// this is a webhook that will be called by github when a new commit is pushed to the website template repository
export async function UpdateTemplateWebhook(req: Request, res: Response) {
  try {
    let tmpDirWebsiteTemplate = process.env.TMP_DIR_WEBSITE_TEMPLATE as string;

    logger.info("UpdateTemplateWebhook", req.body);

    // remove website folder in tmp
    //await fs.remove(tmpDirWebsiteTemplate);

    // if website folder exists, pull, else clone

    if (await fs.pathExists(tmpDirWebsiteTemplate)) {
      let { stdout } = await execPromise(`git pull`, {
        cwd: tmpDirWebsiteTemplate,
      });

      console.log(stdout);
    } else {
      let { stdout } = await execPromise(
        `git clone ${process.env.GIT_WEBSITE_REPOSITORY_URL} ${tmpDirWebsiteTemplate}`
      );

      console.log(stdout);
    }

    // execute npm install

    let { stdout: installOutput } = await execPromise(`npm install`, {
      cwd: tmpDirWebsiteTemplate,
    });

    console.log(installOutput);

    res.status(200).send({ message: "ok" });
  } catch (error) {
    console.log("error", error);
    res.status(500).send({ error: "invalid request" });
  }
}

export async function CreateWebsite(req: Request, res: Response) {
  try {
    let { storeId } = req.body;

    // validate request

    if (!storeId) {
      res.status(400).send({ error: "invalid request" });
      return;
    }

    let storeName = uuidv4();

    storeName = storeName.toLowerCase().replace(/ /g, "-");

    // check if store exists in database

    const websiteExists = await Website.findOne({
      where: {
        store_id: storeId,
      },
    });

    if (websiteExists) {
      res.status(409).send({ error: "website already exists" });
      return;
    }

    // create database record

    await Website.create({
      website_id: storeName,
      store_id: storeId,
    });

    const tmpDirWebsiteTemplate = process.env
      .TMP_DIR_WEBSITE_TEMPLATE as string;
    const tmpDirCustomerWebsite = `${process.env.TMP_DIR}/${storeName}`;
    const customerWebsiteDir = `${process.env.CUSTOMER_WEBSITES_DIR}/${storeName}`;

    // check if website template folder exists

    if (!(await fs.pathExists(tmpDirWebsiteTemplate))) {
      logger.error("Website template folder does not exist. Clone it first.");
      res.status(500).send({ error: "invalid request" });
      return;
    }

    // copy website-template folder to customer website folder
    await fs.copy(tmpDirWebsiteTemplate, tmpDirCustomerWebsite);

    // TODO: add config file

    // run npm build

    let { stdout: buildOutput } = await execPromise(`npm run build`, {
      cwd: tmpDirCustomerWebsite,
    });

    console.log(buildOutput);

    // copy the build folder to customer websites folder

    await fs.copy(`${tmpDirCustomerWebsite}/build`, customerWebsiteDir);

    // remove tmp folder

    await fs.remove(tmpDirCustomerWebsite);

    res.status(200).send({ message: "oks" });
  } catch (error) {
    console.log("error", error);
    res.status(500).send({ error: "invalid request" });
  }
}

export async function GetWebsite(req: Request, res: Response) {
  try {
    const { storeId } = req.params;

    const website = await Website.findOne({
      where: {
        store_id: storeId,
      },
    });

    if (!website) {
      res.status(404).send({ error: "website not found" });
      return;
    }

    res.status(200).json({ message: "ok" });
  } catch (error) {
    console.log("error", error);
    res.status(500).send({ error: "invalid request" });
  }
}
